
#ifndef __SynapticWrapper_h__
#define __SynapticWrapper_h__

#if _MSC_VER > 1000
#pragma once // if possible speed up compilation time
#endif

#include "SynapticSDK/Include/SynKit.h"

#include "OSCOutput.h"
//class MyOSC; //  fwd declaration

// Local Header Files
#include <math.h>

//-----------------------------------------------------------------------------
class PacketData
//-----------------------------------------------------------------------------
{
public:
  PacketData()
  {
    memset(this, 0, sizeof(*this));
  }

  PacketData(SynPacket *pPacket)
  {
    dwSequenceNumber = pPacket->SequenceNumber();
    dwTimestamp = pPacket->TimeStamp();
    lW = pPacket->W();
    lX = pPacket->X();
    lY = pPacket->Y();
    lZ = pPacket->Z();
    lXDelta = pPacket->XDelta();
    lYDelta = pPacket->YDelta();
    lZDelta = pPacket->ZDelta();
    lXMickeys = pPacket->XMickeys();
    lYMickeys = pPacket->YMickeys();
    lXRaw = pPacket->XRaw();
    lYRaw = pPacket->YRaw();
    lZRaw = pPacket->ZRaw();
    lZYRaw = pPacket->ZYRaw();
    dwFingerState = pPacket->FingerState();
    dwExtendedState = pPacket->ExtendedState();
  }

  DWORD dwSequenceNumber;
  DWORD dwTimestamp;
  LONG  lW;
  LONG  lX;
  LONG  lY;
  LONG  lZ;
  LONG  lXDelta;
  LONG  lYDelta;
  LONG  lZDelta;
  LONG  lXMickeys;
  LONG  lYMickeys;
  LONG  lXRaw;
  LONG  lYRaw;
  LONG  lZRaw;
  LONG  lZYRaw;
  DWORD dwState;
  DWORD dwFingerState;
  DWORD dwExtendedState;
};






class PacketBuffer
{
public:
  enum numbers { eSize = 128 };

  PacketBuffer(void)
  {
    m_uiCurrent = 0;
    m_iUnread = 0;
  }

  int NextLocation(void) 
  { 
    m_iUnread += m_iUnread == eSize ? 0 : 1;
    return m_uiCurrent++ % eSize;
  }

  void Add(PacketData &Packet)
  {
    m_Packets[NextLocation()] = Packet;
  }

  PacketData *Read(void)
  {
    if (m_iUnread)
      return m_Packets + ((m_uiCurrent - m_iUnread--) % eSize);
    else
      return 0;
  }

  int Unread(void) { return m_iUnread; }

  PacketData m_Packets[eSize];
  int m_iUnread;
  unsigned int m_uiCurrent;
};






//-----------------------------------------------------------------------------
class CSynapticWrapper 
// Do NOT Derive from Synaptics device events (we haven't a message pump...)
//  NOT >> : public _ISynDeviceEvents << NOT
//-----------------------------------------------------------------------------
{
  ISynAPI    *m_pAPI;
  ISynDevice *m_pDevice;
  SynPacket   m_Packet; // Instantiation will result in CoInitialize();

  PacketBuffer m_Undrawn;

  long m_XMin;
  long m_XMax;
  long m_YMin;
  long m_YMax;

public:

	MyOSC *m_OSCOutput; // no time :]

~CSynapticWrapper()
{
	delete m_pDevice;
	delete m_OSCOutput;
	delete m_pAPI;
}

//-----------------------------------------------------------------------------
CSynapticWrapper(const char *address, int port)
//-----------------------------------------------------------------------------
{
	if(address == NULL || port <= 1024)
	{
		m_OSCOutput = new MyOSC();
	}
	else
	{
	  m_OSCOutput = new MyOSC(address, port);
	}
	OnCreate();
}

//-----------------------------------------------------------------------------
bool OnCreate()
//-----------------------------------------------------------------------------
{
  HRESULT hRes;
	
	hRes = CoInitialize(NULL);

  if( !(hRes == S_OK || hRes == S_FALSE) )
	{
		printf("Error: CoInitialize failed! (err no. %i)\n", hRes);
    return false; // Error other than already initialized on thread.
	}
		

  // Create API interface instance:
	hRes = CoCreateInstance(
					_uuidof(SynAPI),     // Class identifier (CLSID) of the object
					NULL,                // the object is not part of an aggregate
					CLSCTX_INPROC_SERVER,// Context for running executable code
					_uuidof(ISynAPI),    // Reference to the identifier of the interface (riid)
					// Address of output variable that receives the requested interface pointer
					(void **) &m_pAPI);

	if( hRes != S_OK )
	{
		printf("Error: CoCreateInstance failed!\n");
		return false; // Couldn't initialize properly.
	}
	
  // Initialize API interface instance:
  long lHandle = -1;

	hRes =
		  m_pAPI->Initialize() ||
			m_pAPI->FindDevice(SE_ConnectionAny, SE_DeviceTouchPad, &lHandle) || 
			m_pAPI->CreateDevice(lHandle, &m_pDevice);

	if(hRes != 0)
	{
		return false; // Couldn't initialize properly.
	}


  // Setup to receive device packets.
  //m_pDevice->SetSynchronousNotification(this);
	m_pDevice->SetEventNotification(???);


  m_pDevice->GetProperty(SP_XLoSensor, &m_XMin);
  m_pDevice->GetProperty(SP_XHiSensor, &m_XMax);
  m_pDevice->GetProperty(SP_YLoSensor, &m_YMin);
  m_pDevice->GetProperty(SP_YHiSensor, &m_YMax);

  return true;
}

//-----------------------------------------------------------------------------
void TransmitOSC()
//-----------------------------------------------------------------------------
{
  m_pDevice->LoadPacket(m_Packet);

  PacketData sPacket( &m_Packet ); // fetch data into our comfortable struct

			m_OSCOutput->SendValue("/SYN/X", sPacket.lX);
	    m_OSCOutput->SendValue("/SYN/Y", sPacket.lY);
			m_OSCOutput->SendValue("/SYN/Z", sPacket.lZ);
      //pPacket->lXRaw, pPacket->lYRaw, pPacket->lZ);
}

/*
//-----------------------------------------------------------------------------
long STDMETHODCALLTYPE OnSynDevicePacket(long lSeq) // Automatically called
//-----------------------------------------------------------------------------
{
  m_pDevice->LoadPacket(m_Packet);

  PacketData sPacket( &m_Packet ); // fetch data into our comfortable struct

  m_Undrawn.Add(sPacket); // add to packets queue

  return 0;
}


//-----------------------------------------------------------------------------
void TransmitOSC()
//-----------------------------------------------------------------------------
{
PacketData *pPacket;

pPacket = m_Undrawn.Read();

while(pPacket) 
    {
			m_OSCOutput->SendValue("/SYN/X", pPacket->lX);
	    m_OSCOutput->SendValue("/SYN/Y", pPacket->lY);
			m_OSCOutput->SendValue("/SYN/Z", pPacket->lZ);
      //pPacket->lXRaw, pPacket->lYRaw, pPacket->lZ);
			pPacket = m_Undrawn.Read();
    }
		 //Sleep(10);
}
*/

};


#endif

