/*

 Synaptic Touchpad to Open Sound Control
 ...as simple as it is :)

*/

// \o/
#define VC_EXTRALEAN 1
#define WIN32_LEAN_AND_MEAN 1

// Libraries needed for the Synaptic SDK:
#pragma comment(lib,"SynapticSDK/Lib/SynCom.lib")
#pragma comment(lib,"SynapticSDK/Lib/SynCtrl.lib")

// Libraries needed for oscpack:
# pragma comment(lib, "wsock32.lib") // Winsock
# pragma comment(lib, "WS2_32.lib")  // Winsock 2
# pragma comment(lib, "winmm.lib")   // Multimedia (for precision timer)


// Time for some headers...
#include <stdio.h>
#include "OSCOutput.h"
#include "SynapticSDK/Include/SynKit.h"
//#include "CSynapticWrap.h"


#include <conio.h> // 4 kbhit()




int main(int argc, char *argv[])
{
 printf("- Synaptics2OSC v1.0 -\n\n");

if(argc != 3)
{ 
	printf("Usage: %s IP port\n", argv[0]);
	return -1;
}

char *my_ip = argv[1];
int my_port = atol(argv[2]);

printf("\nInitializing OSC output @ %s : %u ...", my_ip, my_port);
MyOSC *my_osc = new MyOSC(my_ip, my_port);
printf("done.\n\n");

//	CSynapticWrapper *my_synaptics = new CSynapticWrapper("localhost", 55555);
//	printf("currently sending OSC data @ %s\n", my_synaptics->m_OSCOutput->m_addressString);


 
  ISynAPI *pAPI = 0;
  HRESULT result = SynCreateAPI(&pAPI);

  if (result)
  {
    printf("Error: Cannot obtain an API object, error: %.8x.\n", result);
    exit(-1);
  }

  // Test some API properties.
  long lValue = 0;
  result = pAPI->GetProperty(SP_Version, &lValue);

  TCHAR tBuf[256];
  long lBufLen = sizeof(tBuf);
  result = pAPI->GetStringProperty(SP_VersionString, tBuf, &lBufLen);

  printf("API version: %.8x.\n", lValue);
  printf("Synaptics Driver version: %s.\n\n", tBuf);

  // Find a device, preferentially a TouchPad or Styk.
  ISynDevice *pDevice = 0;
  long lHandle = -1;
  if ((pAPI->FindDevice(SE_ConnectionAny, SE_DeviceTouchPad, &lHandle) &&
       pAPI->FindDevice(SE_ConnectionAny, SE_DeviceStyk, &lHandle) &&
       pAPI->FindDevice(SE_ConnectionAny, SE_DeviceAny, &lHandle)) || 
       pAPI->CreateDevice(lHandle, &pDevice))
  {
    printf("Error: Unable to find a Synaptics Device.\n");
    exit(-1);
  }

  // Test some device properties.
  result = pDevice->GetProperty(SP_Handle, &lValue);
  result = pDevice->GetProperty(SP_DeviceType, &lValue);
  printf("Found device type: %d,", lValue);
  result = pDevice->GetProperty(SP_ConnectionType, &lValue);
  printf("Connection type: %d.\n", lValue);


  lBufLen = sizeof(tBuf);
  result = pDevice->GetStringProperty(SP_ModelString, tBuf, &lBufLen);
  printf("Device Model ID: %s.\n", tBuf);
  lBufLen = sizeof(tBuf);
  result = pDevice->GetStringProperty(SP_PnPID, tBuf, &lBufLen);
  printf("Device PnP ID: %s.\n", tBuf);
  lBufLen = sizeof(tBuf);
  result = pDevice->GetStringProperty(SP_ShortName, tBuf, &lBufLen);
  printf("Device Config Key: %s.\n", tBuf);

  result = pDevice->GetProperty(SP_DisplayFlags, &lValue);
  if (!(lValue & SF_DisplayPresent))
  {
    printf("Synaptics cPad display not present.\n");
  }

  // Create a unique global named event.
  HANDLE hEvent = 0;

  for (int iInstance = 0; ; iInstance++) 
  {
    char EventName[32];
    sprintf(EventName, "SynCom%d", iInstance);
    hEvent = CreateEvent(0, 0, 0, EventName);

    if (GetLastError() != ERROR_ALREADY_EXISTS)
      break;
  }

  // Find a cPad display if one exists.
  ISynDisplay *pDisplay = 0;
  pDevice->CreateDisplay(&pDisplay);
/*
  if (pDisplay) // If a display was found.
  {
    // Acquire the cPad display, has the side effect of turning on 
    // the backlight.
    result = pDisplay->Acquire(SE_AcquireCooperative); 
    if (result)
    {
      printf("Unable to acquire the cPad.\n");
      exit(-1);
    }

    // Test the backlight.
    result = pDisplay->GetProperty(SP_BackLightState, &lValue);
    printf("cPad backlight state: %d.\n", lValue);
    Sleep(500);

    printf("Turning cPad backlight off.\n");
    result = pDisplay->SetProperty(SP_BackLightState, 0);
    Sleep(500);
    printf("Turning cPad backlight on.\n");
    result = pDisplay->SetProperty(SP_BackLightState, 1); 
  
    // Draw the SDK logo.
    printf("Drawing SDK logo on the cPad display.\n");
    DrawLogo(pDisplay, iInstance);
    pDisplay->FlushDC(SE_FlushSynchronous);
  }
*/

  // Setup notification of API, device, and display events.
  pAPI->SetEventNotification(hEvent);
  pDevice->SetEventNotification(hEvent);
  if (pDisplay) pDisplay->SetEventNotification(hEvent);


	long XMin, XMax;
	long YMin, YMax;
  pDevice->GetProperty(SP_XLoSensor, &XMin);
  pDevice->GetProperty(SP_XHiSensor, &XMax);
  pDevice->GetProperty(SP_YLoSensor, &YMin);
  pDevice->GetProperty(SP_YHiSensor, &YMax);

	printf("(Xmin,Ymin,Xmax,Ymax) = (%i,%i,%i,%i)\n\n", XMin, YMin, XMax, YMax);


  printf("Entering packet processing loop.\n");
  printf("Tap on the device or press a device button to exit the program.\n");




  // Loop until a device packet that contains a button or tap event occurs.
  SynPacket Packet;

  for (;;)
  {
    WaitForSingleObject(hEvent, INFINITE);
    
    for (;;)
    {
      long lParameter = 0;
      result = pAPI->GetEventParameter(&lParameter);

      if (result != S_OK) break;

       printf("API event parameter: %d.\n", lParameter);
    }

    for (; pDisplay; )
    {
      long lParameter = 0;
      result = pDisplay->GetEventParameter(&lParameter);
      if (result != S_OK)
        break;

      printf("Display event parameter: %d.\n", lParameter);

      if (lParameter == SE_AcquisitionGained)
      {
        //DrawLogo(pDisplay, iInstance);
        //pDisplay->FlushDC(SE_FlushSynchronous);
        // Wait for taps or buttons on the terminating application 
        // to complete before restoring notification.
        //Sleep(100);
        pDevice->SetEventNotification(hEvent);
      }
      else
      {
        // Another application has acquired the display, turn off notification 
        // so we don't terminate due to a tap on that app.
        //pDevice->SetEventNotification(0);
      }
    }

    for ( ; ; )
    {
      result = pDevice->LoadPacket(Packet);
      if (result == SYNE_FAIL) break;

			
		
			if ( Packet.FingerState() & SF_FingerTouch )
			{
      //printf("Sequence number: %d, Finger state: %.8x.\n", 
      //  Packet.SequenceNumber(), Packet.FingerState());
			my_osc->SendValue("/SYN/X", (Packet.X()-XMin)/(float)(XMax-XMin));
			my_osc->SendValue("/SYN/Y", (Packet.Y()-YMin)/(float)(YMax-YMin));
			my_osc->SendValue("/SYN/Z", 3.0*(Packet.Z()-4.0)/(255.0-4.0));
			} else
			{
				my_osc->SendValue("/SYN/Z", 0.0);
			}


			
      //if ((Packet.FingerState() & SF_FingerTap) || Packet.ButtonState())
			//if (Packet.ButtonState())
			if(kbhit())
      {
        exit(0);
      }
    }
  }

  if (pDisplay)
  {
    pDisplay->Unacquire(); 
    pDisplay->Release();
  }

  pDevice->Release();
  pAPI->Release();

	delete my_osc;

  return 0;
}

